*Batch Design with Storage Tanks
*20 products/12 stages/10 potential tanks

$TITLE BATCH PROCESSING PROBLEM WITH INTERMEDIATE STORAGE TANKS
$OFFSYMXREF
$OFFSYMLIST
*
*  MINLP FORMULATION OF BATCH PROCESSING PROBLEM

 SETS         I         PRODUCTS               / A , B , C , D , E, F, G, H, HH, HHH, AA , AB, AC, AD, AE, AF, AG, AH, AHH, AHHH /
              J         POTENTIAL STAGES                   / 1 * 12 /
              M         POTENTIAL STORAGE TANKS            / 1 * 10 /
              K         BINARIES NEEDED TO REPRESENT N(J)  / 1 * 10 /
              D                                            / 1 * 2/;

ALIAS (J,KK);

 SCALAR       H         HORIZON TIME (available time hrs)  / 6000. /
              ALPHA     COST COEFFICIENT FOR BATCH UNITS   / 250.  /
              COSST     COST COEFF. FOR STORAGE TANK       /150./
              GAMMA     COST COEFF. FOR STORAGE TANK       /0.5 /

 PARAMETERS   Q(I)      PRODUCTION RATE OF PRODUCT i    / A=250000 ,
                         B=150000 , C=180000 , D=160000 , E=120000, F=130000, G=190000, H = 140000, HH = 175000, HHH = 125000, AA = 140000, AB = 220000,
                         AC = 300000, AD = 200000, AE = 120000, AF = 320000, AG = 400500, AH = 210000, AHH = 310000, AHHH = 70000/
              PHI
              ACTTL(I,J)
              ACTV(J)   ACTUAL VOLUME
              ACTB(I,J) ACTUAL BATCH SIZES
              ACTN(J)   ACTUAL NUMBER OF UNITS IN PARALLEL  (in phase)
              COEFF(K) ;
              COEFF('1') = LOG(1.)  ;
              COEFF('2') = LOG(2.)  ;
              COEFF('3') = LOG(3.)  ;
              COEFF('4') = LOG(4.)  ;
              COEFF('5') = LOG(5.)  ;
              COEFF('6') = LOG(6.)  ;
              COEFF('7') = LOG(7.)  ;
              COEFF('8') = LOG(8.)  ;
              COEFF('9') = LOG(9.)  ;
              COEFF('10') = LOG(10.)  ;


 TABLE        S(I,J)    SIZE FACTOR FOR PRODUCT i IN STAGE j

               1         2         3         4         5         6        7        8        9       10      11    12
    A         2.9       2.        5.2       4.9       6.1       4.2       5.0      1.1      3.2     4.5     1.7   1.3
    B         0.8       0.8       0.9       3.4       2.1       2.5       0.9      2.2      1.4     2.2     2.6   4.3
    C         0.7       2.6       1.6       3.6       3.2       2.9       3.6      4.0      4.3     0.6     2.5   5.3
    D         4.7       2.3       1.6       2.7       1.2       2.5       3.9      0.6      3.1     5.8     3.7   6.5
    E         1.2       3.6       2.4       4.5       1.6       2.1       0.9      4.2      2.1     4.1     4.4   7.7
    F         3.1       2.5       4.5       1.3       3.3       4.1       1.4      1.0      3.5     0.7     2.2   3.8
    G         1.0       2.2       0.7       4.2       2.8       2.3       5.2      1.9      2.6     1.1     4.7   4.9
    H         2.5       1.1       5.3       2.6       3.7       4.0       2.2      3.3      2.7     3.4     6.0   6.9
   HH         2.4       2.5       2.7       3.2       2.3       1.7       5.2      4.7      2.6     1.4     3.7   3.6
  HHH         2.7       3.3       4.4       2.6       3.7       0.5       2.7      6.2      3.2     3.4     6.5   6.0
   AA         2.9       2.        5.2       4.9       6.1       4.2       5.0      1.1      3.2     4.5     4.3   3.5
   AB         0.8       0.8       0.9       3.4       2.1       2.5       0.9      2.2      1.4     2.2     3.2   6.1
   AC         1.2       3.6       2.4       4.5       1.6       2.1       0.9      4.2      2.1     4.1     4.4   7.7
   AD         3.1       2.5       4.5       1.3       3.3       4.1       1.4      1.0      3.5     0.7     2.2   3.8
   AE         1.0       2.2       0.7       4.2       2.8       2.3       5.2      1.9      2.6     1.1     4.7   4.9
   AF         1.0       2.2       0.7       4.2       2.8       2.3       5.2      1.9      2.6     1.1     4.7   4.9
   AG         2.5       1.1       5.3       2.6       3.7       4.0       2.2      3.3      2.7     3.4     6.0   6.9
   AH         2.4       2.5       2.7       3.2       2.3       1.7       5.2      4.7      2.6     1.4     3.7   3.6
  AHH         2.7       3.3       4.4       2.6       3.7       0.5       2.7      6.2      3.2     3.4     6.5   6.0
 AHHH         2.9       2.        5.2       4.9       6.1       4.2       5.0      1.1      3.2     4.5     4.3   3.5


 TABLE        T(I,J)    PROCESSING TIME OF PRODUCT I IN BATCH J

               1         2         3         4         5         6        7        8        9       10      11    12
    A         6.4       4.7       1.3       3.9       2.1       1.6       3.2      3.1      4.2     5.2     5.6   8.9
    B         3.8       6.4       6.5       4.4       2.3       3.2       5.2      2.5      4.4     1.1     4.5   6.7
    C         1.0       6.3       3.4       4.9       2.7       6.2       3.1      1.2      2.3     5.1     3.7   5.6
    D         3.2       3.0       3.5       3.3       2.8       3.4       4.2      2.9      6.2     2.2     5.8   4.5
    E         2.1       2.5       4.2       3.6       3.7       2.2       5.1      0.4      4.1     1.3     6.6   3.4
    F         3.5       4.1       0.9       1.4       3.6       2.7       2.9      3.7      0.8     1.5     4.6   3.3
    G         4.1       6.7       2.2       1.4       0.7       4.7       1.3      0.6      3.2     2.1     3.4   2.6
    H         5.3       3.1       2.8       1.5       3.2       6.1       0.5      3.9      2.7     4.1     6.6   1.6
   HH         2.6       5.2       3.2       3.4       4.4       1.1       7.1      2.5      3.1     0.9     7.8   5.8
  HHH         1.7       5.2       3.7       2.6       5.2       2.6       5.1      8.2      1.1     2.9     8.4   4.8
   AA         6.4       4.7       1.3       3.9       2.1       1.6       3.2      3.1      4.2     5.2     9.3   6.5
   AB         3.8       6.4       6.5       4.4       2.3       3.2       5.2      2.5      4.4     1.1     0.2   6.4
   AC         3.5       4.1       0.9       1.4       3.6       2.7       2.9      3.7      0.8     1.5     4.6   3.3
   AD         4.1       6.7       2.2       1.4       0.7       4.7       1.3      0.6      3.2     2.1     3.4   2.6
   AE         5.3       3.1       2.8       1.5       3.2       6.1       0.5      3.9      2.7     4.1     6.6   1.6
   AF         3.8       6.4       6.5       4.4       2.3       3.2       5.2      2.5      4.4     1.1     4.5   6.7
   AG         1.0       6.3       3.4       4.9       2.7       6.2       3.1      1.2      2.3     5.1     3.7   5.6
   AH         3.2       3.0       3.5       3.3       2.8       3.4       4.2      2.9      6.2     2.2     5.8   4.5
  AHH         2.1       2.5       4.2       3.6       3.7       2.2       5.1      0.4      4.1     1.3     6.6   3.4
 AHHH         3.5       4.1       0.9       1.4       3.6       2.7       2.9      3.7      0.8     1.5     4.6   3.3


 VARIABLES    V(J)      VOLUME OF STAGE J
              BAT(I)    BATCH SIZE of PRODUCT i
              B(I,J)    BATCH SIZE OF PRODUCT i STAGE(j)
              TL(I,J)     CYCLE TIME OF PRODUCT i
              N(J)      NUMBER OF UNITS IN PARALLEL STAGE j
              MI(J)
              COST      TOTAL COST OF BATCH PROCESSING UNITS
              VST(J)    VOLUME OF STORAGE TANK BETEWN STAGE J AND J+1
              E(I) ;
 BINARY   VARIABLES     Y1(K,J), Y2(K,J), Y3(J)                             ;
 POSITIVE VARIABLES     V(J), B(I,J), BAT(I), TL(I,J) , N(J),  MI(J), VST(J);

EQUATIONS    VOL(I,J)     CALCULATE VOLUME OF STAGE j
              TI(I,J)
*             CYCLE(I,J)
              TIME         TIME CONSTRAINT
              VOLUM1(I,J) STORAGE TANK SIZE EQUATION for PRODUCT I STAGE J
              VOLUM2(I,J) STORAGE TANK SIZE EQUATION for PRODUCT I STAGE J
              UNIT1_1(J,K)
              UNIT1_2(J,K)
              UNIT2_1(J,K)
              UNIT2_2(J,K)
              LIM1(J)    LIMIT SELECTION TO ONE NUMBER for N(J)
              LIM2(J)
              BTSZE1(I,J) BATCH SIZE EQUAT. PRODUCT I stage J
              BTSZE2(I,J) BATCH SIZE EQUAT. PRODUCT i stage J
              OBJ       OBJECTIVE FUNCTION DEFINITION
              VOLUM1_2(J)
              VOLUM2_2(J)
              BTSZE1_2(I,J)
              BTSZE2_2(I,J);

*******************************************************************************
*  BOUNDS SECTION
 V.LO(J)   =  LOG(300.) ;
 V.UP(J)   =  LOG(3500.);
 VST.LO(J)   =  LOG(100.) ;
 VST.UP(J)   = LOG(15000.);
 N.UP(J)   =  LOG(10.)   ;
 MI.UP(J)   =  LOG(10.)   ;
* COST.lo = 0.0      ;
 PARAMETERS   TLLOW(I)       LOWER BOUND ON TL(I)
              TLUPP(I)       UPPER BOUND ON TL(I)                    ;
              TLLOW(I)  =  SMAX(J, T(I,J) / EXP(N.UP(J)))            ;
              TLUPP(I)  =  SMAX(J, T(I,J) )                          ;
 TL.LO(I,J)  =  LOG(TLLOW(I)) ;
 TL.UP(I,J)  =  LOG(TLUPP(I)) ;
 PARAMETERS   BLOW(I)        LOWER BOUND ON B
              BUPP(I)        UPPER BOUND ON B                    ;

              BLOW(I)  =  Q(I) * ( SMAX(J, T(I,J) / EXP(N.UP(J))))/ H;

              BUPP(I)  =  MIN( Q(I) , SMIN(J , EXP(V.UP(J))/S(I,J))) ;

 B.LO(I,J)   =  LOG(BLOW(I)) ;
 B.UP(I,J)   =  LOG(BUPP(I)) ;

 PARAMETERS   M1(K) /1*10 = 0/
              M2(K) /1*10 = 0/;
              M1(K) = LOG(10) - COEFF(K);
              M2(K) = COEFF(K) - LOG(1) ;

************************************************************************************
 OBJ..          COST  =E=  ALPHA * SUM(J ,EXP(N(J) + MI(J) + 0.6*V(J))) +
                           150.  * SUM(J $(ORD(J) LT CARD(J)),EXP(0.5 * VST(J)));

 VOL(I,J)..     V(J) =G= LOG(S(I,J)) + B(I,J)  - N(J)                   ;
* CYCLE(I,J)..   MI(J) + TL(I,J)  =G=  LOG(T(I,J))                      ;
 TI(I,J)..      E(I) =G= LOG(T(I,J))- B(I,J)- MI(J)                     ;
 TIME..         SUM(I , Q(I) * EXP(E(I))) =L= H                         ;

 VOLUM1(I,J)$(ORD(J) LT CARD(J))..  VST(J) =G=LOG(2*5)+B(I,J+1)-(LOG(BUPP(I))-LOG(100)+LOG(10))*(1-Y3(J))    ;
 VOLUM2(I,J)$(ORD(J) LT CARD(J))..  VST(J) =G=LOG(2*5)+B(I,J)-(LOG(BUPP(I))-LOG(100)+LOG(10))*(1-Y3(J))      ;
 BTSZE1(I,J)$(ORD(J) LT CARD(J))..  B(I,J) - B(I,J+1) =L= LOG(3.)+ (LOG(BUPP(I))-LOG(BLOW(I))-LOG(3))*(1-Y3(J))        ;
 BTSZE2(I,J)$(ORD(J) LT CARD(J))..  B(I,J) - B(I,J+1) =G= -LOG(3.)- (LOG(BUPP(I))-LOG(BLOW(I))-LOG(3))*(1-Y3(J))       ;

 VOLUM1_2(J)$(ORD(J) LT CARD(J))..    VST(J) =L= -LOG(10000)+(LOG(15000)+LOG(10000))*Y3(J)    ;
 VOLUM2_2(J)$(ORD(J) LT CARD(J))..    VST(J) =G= -LOG(10000)-(LOG(100)+LOG(10000))*Y3(J)  ;
 BTSZE1_2(I,J)$(ORD(J) LT CARD(J))..  B(I,J) - B(I,J+1) =L= (LOG(BUPP(I))-LOG(BLOW(I)))*Y3(J)        ;
 BTSZE2_2(I,J)$(ORD(J) LT CARD(J))..  B(I,J) - B(I,J+1) =G= - (LOG(BUPP(I))-LOG(BLOW(I)))*Y3(J)       ;

 UNIT1_1(J,K)..     N(J)  =L=       COEFF(K) + M1(K)*(1-Y1(K,J))         ;
 UNIT1_2(J,K)..     N(J)  =G=       COEFF(K) - M2(K)*(1-Y1(K,J))         ;
 LIM1(J)..        SUM(K , Y1(K,J)) =E= 1                               ;
 UNIT2_1(J,K)..       MI(J)  =L=      COEFF(K) + M1(K)*(1-Y2(K,J))         ;
 UNIT2_2(J,K)..     MI(J)  =G=      COEFF(K) - M2(K)*(1-Y2(K,J))         ;
 LIM2(J)..        SUM(K , Y2(K,J)) =E= 1                               ;


 MODEL   BATCH_BM   / ALL / ;

 OPTION MINLP = SBB;
OPTION RMINLP = CONOPT;
OPTION MIP = CPLEX;
OPTION ITERLIM = 100000000;
OPTION RESLIM = 10800;
OPTION OPTCR    = 0.0 ;
OPTION OPTCA    = 0.0 ;
OPTION LIMROW   = 0 ;
OPTION LIMCOL   = 0 ;
*OPTION SOLPRINT = OFF ;
*OPTION SYSOUT   = OFF ;

* SOLVE BATCH USING RMINLP  MINIMIZING  COST ;

BATCH_BM.OPTFILE = 1;
BATCH_BM.NODLIM = 1000000;
SOLVE BATCH_BM USING MINLP  MINIMIZING  COST ;

PARAMETERS ACTVST(J), ACTMI(J), ACTE(I);
PARAMETERS TLIJ(I,J),TLI(I);
 ACTE(I) = EXP(E.L(I));
 ACTV(J) = EXP(V.L(J))      ;
 ACTB(I,J) = EXP(B.L(I,J))  ;
 ACTTL(I,J) = EXP(TL.L(I,J))    ;
 TLIJ(I,J) = ACTE(I)*ACTB(I,J);
 TLI(I) = SMAX(J,TLIJ(I,J));
 ACTN(J) = EXP(N.L(J))  ;
 ACTMI(J) = EXP(MI.L(J));
 ACTVST(J) = EXP(VST.L(J));
 DISPLAY ACTV       ;
 DISPLAY ACTVST     ;
 DISPLAY ACTB,ACTE       ;
 DISPLAY ACTTL,TLIJ,TLI      ;
 DISPLAY ACTN       ;
 DISPLAY ACTMI      ;
* DISPLAY PHI;